[dpdk-dev,v2] net/ixgbe: remove MTU setting limitation when port's stopped

Message ID 1499222024-64581-1-git-send-email-beilei.xing@intel.com (mailing list archive)
State Superseded, archived
Delegated to: Ferruh Yigit
Headers

Checks

Context Check Description
ci/checkpatch success coding style OK
ci/Intel-compilation success Compilation OK

Commit Message

Xing, Beilei July 5, 2017, 2:33 a.m. UTC
  Currently, setting MTU will fail if MTU requires the support
of scattered packets before scatter is enabled.
This patch allows setting this special MTU when device is
stopped, cause scatter will be re-configured during starting
port.

Signed-off-by: Beilei Xing <beilei.xing@intel.com>
---
v2 changes:
 - change commit log

 drivers/net/ixgbe/ixgbe_ethdev.c | 11 +++++++----
 1 file changed, 7 insertions(+), 4 deletions(-)
  

Comments

Wenzhuo Lu July 5, 2017, 3:19 a.m. UTC | #1
Hi,


> -----Original Message-----
> From: Xing, Beilei
> Sent: Wednesday, July 5, 2017 10:34 AM
> To: Lu, Wenzhuo
> Cc: dev@dpdk.org
> Subject: [PATCH v2] net/ixgbe: remove MTU setting limitation when port's
> stopped
> 
> Currently, setting MTU will fail if MTU requires the support of scattered
> packets before scatter is enabled.
> This patch allows setting this special MTU when device is stopped, cause
> scatter will be re-configured during starting port.
> 
> Signed-off-by: Beilei Xing <beilei.xing@intel.com>
Acked-by: Wenzhuo Lu <wenzhuo.lu@intel.com>
  
Ferruh Yigit July 5, 2017, 1:59 p.m. UTC | #2
On 7/5/2017 3:33 AM, Beilei Xing wrote:
> Currently, setting MTU will fail if MTU requires the support
> of scattered packets before scatter is enabled.
> This patch allows setting this special MTU when device is
> stopped, cause scatter will be re-configured during starting
> port.
> 
> Signed-off-by: Beilei Xing <beilei.xing@intel.com>
> ---
> v2 changes:
>  - change commit log
> 
>  drivers/net/ixgbe/ixgbe_ethdev.c | 11 +++++++----
>  1 file changed, 7 insertions(+), 4 deletions(-)
> 
> diff --git a/drivers/net/ixgbe/ixgbe_ethdev.c b/drivers/net/ixgbe/ixgbe_ethdev.c
> index fb5574e..ece5d6c 100644
> --- a/drivers/net/ixgbe/ixgbe_ethdev.c
> +++ b/drivers/net/ixgbe/ixgbe_ethdev.c
> @@ -4812,6 +4812,7 @@ ixgbe_dev_mtu_set(struct rte_eth_dev *dev, uint16_t mtu)
>  	struct rte_eth_dev_info dev_info;
>  	uint32_t frame_size = mtu + ETHER_HDR_LEN + ETHER_CRC_LEN;
>  	struct rte_eth_rxmode *rx_conf = &dev->data->dev_conf.rxmode;
> +	struct rte_eth_dev_data *dev_data = dev->data;
>  
>  	ixgbe_dev_info_get(dev, &dev_info);
>  
> @@ -4819,13 +4820,15 @@ ixgbe_dev_mtu_set(struct rte_eth_dev *dev, uint16_t mtu)
>  	if ((mtu < ETHER_MIN_MTU) || (frame_size > dev_info.max_rx_pktlen))
>  		return -EINVAL;
>  
> -	/* refuse mtu that requires the support of scattered packets when this
> -	 * feature has not been enabled before.
> +	/* If device is started, refuse mtu that requires the support of
> +	 * scattered packets when this feature has not been enabled before.
>  	 */
> -	if (!rx_conf->enable_scatter &&
> +	if (dev_data->dev_started && !rx_conf->enable_scatter &&

Should dev->data->scattered_rx be used here?

rx_conf->enable_scatter is configuration value provided by user, but
dev->data->scattered_rx keeps the actual status of the feature.
Driver may decide to enable scattered rx even tough configuration didn't
asked for it, this behavior is what this patch relies on.

Can you confirm that there is no other case that can prevent enabling
scattered rx?
Otherwise this patch will let setting MTU bigger than mbuf and if
somehow scattered rx is not enabled, this may break the Rx path I guess.

>  	    (frame_size + 2 * IXGBE_VLAN_TAG_SIZE >
> -	     dev->data->min_rx_buf_size - RTE_PKTMBUF_HEADROOM))
> +	     dev->data->min_rx_buf_size - RTE_PKTMBUF_HEADROOM)) {
> +		PMD_INIT_LOG(ERR, "Stop port first.");
>  		return -EINVAL;
> +	}
>  
>  	hw = IXGBE_DEV_PRIVATE_TO_HW(dev->data->dev_private);
>  	hlreg0 = IXGBE_READ_REG(hw, IXGBE_HLREG0);
>
  
Xing, Beilei July 6, 2017, 3:23 a.m. UTC | #3
> -----Original Message-----

> From: dev [mailto:dev-bounces@dpdk.org] On Behalf Of Ferruh Yigit

> Sent: Wednesday, July 5, 2017 10:00 PM

> To: Xing, Beilei <beilei.xing@intel.com>; Lu, Wenzhuo

> <wenzhuo.lu@intel.com>

> Cc: dev@dpdk.org

> Subject: Re: [dpdk-dev] [PATCH v2] net/ixgbe: remove MTU setting

> limitation when port's stopped

> 

> On 7/5/2017 3:33 AM, Beilei Xing wrote:

> > Currently, setting MTU will fail if MTU requires the support of

> > scattered packets before scatter is enabled.

> > This patch allows setting this special MTU when device is stopped,

> > cause scatter will be re-configured during starting port.

> >

> > Signed-off-by: Beilei Xing <beilei.xing@intel.com>

> > ---

> > v2 changes:

> >  - change commit log

> >

> >  drivers/net/ixgbe/ixgbe_ethdev.c | 11 +++++++----

> >  1 file changed, 7 insertions(+), 4 deletions(-)

> >

> > diff --git a/drivers/net/ixgbe/ixgbe_ethdev.c

> > b/drivers/net/ixgbe/ixgbe_ethdev.c

> > index fb5574e..ece5d6c 100644

> > --- a/drivers/net/ixgbe/ixgbe_ethdev.c

> > +++ b/drivers/net/ixgbe/ixgbe_ethdev.c

> > @@ -4812,6 +4812,7 @@ ixgbe_dev_mtu_set(struct rte_eth_dev *dev,

> uint16_t mtu)

> >  	struct rte_eth_dev_info dev_info;

> >  	uint32_t frame_size = mtu + ETHER_HDR_LEN + ETHER_CRC_LEN;

> >  	struct rte_eth_rxmode *rx_conf = &dev->data->dev_conf.rxmode;

> > +	struct rte_eth_dev_data *dev_data = dev->data;

> >

> >  	ixgbe_dev_info_get(dev, &dev_info);

> >

> > @@ -4819,13 +4820,15 @@ ixgbe_dev_mtu_set(struct rte_eth_dev *dev,

> uint16_t mtu)

> >  	if ((mtu < ETHER_MIN_MTU) || (frame_size >

> dev_info.max_rx_pktlen))

> >  		return -EINVAL;

> >

> > -	/* refuse mtu that requires the support of scattered packets when

> this

> > -	 * feature has not been enabled before.

> > +	/* If device is started, refuse mtu that requires the support of

> > +	 * scattered packets when this feature has not been enabled before.

> >  	 */

> > -	if (!rx_conf->enable_scatter &&

> > +	if (dev_data->dev_started && !rx_conf->enable_scatter &&

> 

> Should dev->data->scattered_rx be used here?


Agree, I missed this case: if launch testpmd with max_pkt_len bigger than mbuf.
Thanks.

> 

> rx_conf->enable_scatter is configuration value provided by user, but

> dev->data->scattered_rx keeps the actual status of the feature.

> Driver may decide to enable scattered rx even tough configuration didn't

> asked for it, this behavior is what this patch relies on.

> 

> Can you confirm that there is no other case that can prevent enabling

> scattered rx?


I checked that scattered_rx is disabled only when device is stopped.

> Otherwise this patch will let setting MTU bigger than mbuf and if somehow

> scattered rx is not enabled, this may break the Rx path I guess.

> 

> >  	    (frame_size + 2 * IXGBE_VLAN_TAG_SIZE >

> > -	     dev->data->min_rx_buf_size - RTE_PKTMBUF_HEADROOM))

> > +	     dev->data->min_rx_buf_size - RTE_PKTMBUF_HEADROOM)) {

> > +		PMD_INIT_LOG(ERR, "Stop port first.");

> >  		return -EINVAL;

> > +	}

> >

> >  	hw = IXGBE_DEV_PRIVATE_TO_HW(dev->data->dev_private);

> >  	hlreg0 = IXGBE_READ_REG(hw, IXGBE_HLREG0);

> >
  

Patch

diff --git a/drivers/net/ixgbe/ixgbe_ethdev.c b/drivers/net/ixgbe/ixgbe_ethdev.c
index fb5574e..ece5d6c 100644
--- a/drivers/net/ixgbe/ixgbe_ethdev.c
+++ b/drivers/net/ixgbe/ixgbe_ethdev.c
@@ -4812,6 +4812,7 @@  ixgbe_dev_mtu_set(struct rte_eth_dev *dev, uint16_t mtu)
 	struct rte_eth_dev_info dev_info;
 	uint32_t frame_size = mtu + ETHER_HDR_LEN + ETHER_CRC_LEN;
 	struct rte_eth_rxmode *rx_conf = &dev->data->dev_conf.rxmode;
+	struct rte_eth_dev_data *dev_data = dev->data;
 
 	ixgbe_dev_info_get(dev, &dev_info);
 
@@ -4819,13 +4820,15 @@  ixgbe_dev_mtu_set(struct rte_eth_dev *dev, uint16_t mtu)
 	if ((mtu < ETHER_MIN_MTU) || (frame_size > dev_info.max_rx_pktlen))
 		return -EINVAL;
 
 	/* refuse mtu that requires the support of scattered packets when this
 	 * feature has not been enabled before.
 	 */
-	if (!rx_conf->enable_scatter &&
+	if (dev_data->dev_started && !rx_conf->enable_scatter &&
 	    (frame_size + 2 * IXGBE_VLAN_TAG_SIZE >
-	     dev->data->min_rx_buf_size - RTE_PKTMBUF_HEADROOM))
+	     dev->data->min_rx_buf_size - RTE_PKTMBUF_HEADROOM)) {
+		PMD_INIT_LOG(ERR, "Stop port first.");
 		return -EINVAL;
+	}
 
 	hw = IXGBE_DEV_PRIVATE_TO_HW(dev->data->dev_private);
 	hlreg0 = IXGBE_READ_REG(hw, IXGBE_HLREG0);
@@ -4819,13 +4820,15 @@  ixgbe_dev_mtu_set(struct rte_eth_dev *dev, uint16_t mtu)
 		return -EINVAL;
 
-	/* refuse mtu that requires the support of scattered packets when this
-	 * feature has not been enabled before.
+	/* If device is started, refuse mtu that requires the support of
+	 * scattered packets when this feature has not been enabled before.
 	 */